#include "postgresql-libpq-6.2.x.hh"

//
//  UESQLC::postgresql-libpq-6.2.x internal variables initiation.
//

string      TPostgresql62xManager::_ySqlStatement;
string      TPostgresql62xManager::_yHost;
PGconn*     TPostgresql62xManager::_ptPgconn      = NULL;
PGresult*   TPostgresql62xManager::_ptPgresult    = NULL;
PGresult*   TPostgresql62xManager::_ptAuxPgresult = NULL;
string      TPostgresql62xManager::_yDatabaseName = getenv ("USER");


//
//  C O N S T R U C T O R S
//

int TPostgresql62xManager::initialize (void)
{

  int   iSuccess = gIsThereConnection;

  if ( !gIsThereConnection )
  {
    _ptPgconn = PQsetdb (NULL, NULL, NULL, NULL, getenv ("USER"));
    if ( PQstatus (_ptPgconn) == CONNECTION_BAD )
    {
      //
      //  Condition:           connection exception
      //  Class subcondition:  SQL-client unable to establish SQL-connection
      //
      uesql::SQLCODE  = -1;
      uesql::SQLSTATE = "08001";
    }
    else
    {
      uesql::SQLCODE  = 0;
      uesql::SQLSTATE = "00000";
      iSuccess        = 1;
      connect ("DEFAULT", _ptPgconn);
    }
  }
  return iSuccess;

}  // initialize()


void TPostgresql62xManager::connect (const string& rkyCONNECTION_NAME, const PGconn* pktCONNECTION)
{

  xConnectionMap.bind (rkyCONNECTION_NAME, (PGconn*) pktCONNECTION);
  yCurrentConnection = rkyCONNECTION_NAME;
  gIsThereConnection = true;

}  // connect()


void TPostgresql62xManager::disconnect (const string& rkyCONNECTION_NAME)
{

  PQfinish (xConnectionMap [rkyCONNECTION_NAME]);
  xConnectionMap.erase (rkyCONNECTION_NAME);
  if ( rkyCONNECTION_NAME == yCurrentConnection )
  {
    yCurrentConnection.erase();
    gIsThereConnection = false;
  }

}  // disconnect()


void TPostgresql62xManager::disconnectCurrent (void)
{

  disconnect (yCurrentConnection);

}  // disconnectCurrent()


PGconn* TPostgresql62xManager::setCurrentConnection (const string& rkyCONNECTION_NAME)
{

  PGconn*   ptConnectionPgconn = NULL;

  if ( xConnectionMap.is_binded (rkyCONNECTION_NAME) )
  {
    yCurrentConnection = rkyCONNECTION_NAME;
    ptConnectionPgconn = xConnectionMap [rkyCONNECTION_NAME];
    gIsThereConnection = true;
  }
  return ptConnectionPgconn;

}  // setCurrentConnection()


//
//  S E L E C T O R S
//

bool TPostgresql62xManager::existsDefaultConnection (void) const
{

  return !yCurrentConnection.empty();

}  // existsDefaultConnection()


bool TPostgresql62xManager::isConnected (const string& rkyCONNECTION_NAME) const
{

  return xConnectionMap.is_binded (rkyCONNECTION_NAME);

}  // isConnected()


PGconn* TPostgresql62xManager::currentConnection (void) const
{

  PGconn*   ptCurrentPgconn = NULL;

  if ( !yCurrentConnection.empty() )
  {
    ptCurrentPgconn = xConnectionMap [yCurrentConnection];
  }
  return ptCurrentPgconn;

}  // currentConnection()


PGconn* TPostgresql62xManager::connection (const string& rkyCONNECTION_NAME) const
{

  return xConnectionMap [rkyCONNECTION_NAME];

}  // connection()


void TPostgresql62xManager::finish (void)
{

  register mpcl::xextended_map<string, PGconn*>::iterator   tIter = xConnectionMap.begin();

  for (; ( tIter != xConnectionMap.end() ) ;tIter++)
  {
    PQfinish (tIter->second);
  }
  xConnectionMap.clear();
  yCurrentConnection.erase();

}  // finish()


//
//  UESQLC::postgresql-libpq-6.2.x variables declaration.
//

TPostgresql62xManager   htPostgresql62xManager;
